@@ -150,6 +150,7 @@ leaflet_sfc <- function(x,
                         ...) {
   if (!is.null(names(x))) names(x) = NULL
   if (is_literally_false(highlight)) highlight = NULL
+  if (is_literally_false(popup)) popup = NULL
   if (inherits(x, "XY")) x = sf::st_sfc(x)
   if (!native.crs) x <- checkAdjustProjection(x)
   if (is.na(sf::st_crs(x)$proj4string)) native.crs <- TRUE
@@ -347,7 +348,7 @@ featureComplexity = function(x) {
       getGeometryType(x),
       "pt" = nNodes(x) / 1e6 * dm[1] * dm[2],
       "ln" = nNodes(x) / 1e6 * dm[1] * dm[2],
-      "pl" = nNodes(x) / 1e6 * nrings(x) * dm[1] * dm[2],
+      "pl" = nNodes(x) / 1e6 * nrings(x) * dm[2],
       "gc" = nNodes(x) / 1e6 * dm[1] * dm[2]
     )
   } else {
@@ -355,7 +356,7 @@ featureComplexity = function(x) {
       getGeometryType(x),
       "pt" = nNodes(x) / 1e6 * length(x),
       "ln" = nNodes(x) / 1e6 * length(x),
-      "pl" = nNodes(x) / 1e6 * nrings(x) * length(x),
+      "pl" = nNodes(x) / 1e6 * nrings(x),
       "gc" = nNodes(x) / 1e6 * length(x)
     )
   }
