@@ -15,9 +15,9 @@ if ( !isGeneric('viewRGB') ) {
 #' from \code{\link{plotRGB}}.
 #'
 #' @param x a RasterBrick or RasterStack
-#' @param r integer. Index of the Red channel, between 1 and nlayers(x)
-#' @param g integer. Index of the Green channel, between 1 and nlayers(x)
-#' @param b integer. Index of the Blue channel, between 1 and nlayers(x)
+#' @param r integer. Index of the Red channel/band, between 1 and nlayers(x)
+#' @param g integer. Index of the Green channel/band, between 1 and nlayers(x)
+#' @param b integer. Index of the Blue channel/band, between 1 and nlayers(x)
 #' @param quantiles the upper and lower quantiles used for color stretching. If set to NULL, no stretching is applied.
 #' @param map the map to which the layer should be added
 #' @param maxpixels integer > 0. Maximum number of cells to use for the plot.
@@ -39,10 +39,12 @@ if ( !isGeneric('viewRGB') ) {
 #' Tim Appelhans
 #'
 #' @examples
+#' \dontrun{
 #' library(raster)
 #'
 #' viewRGB(poppendorf, 4, 3, 2) # true-color
 #' viewRGB(poppendorf, 5, 4, 3) # false-color
+#' }
 #'
 #' @export
 #' @docType methods
@@ -97,15 +99,85 @@ setMethod("viewRGB", signature(x = "RasterStackBrick"),
             lyrs <- paste(r, g, b, sep = ".")
             grp <- paste(grp, lyrs, sep = "_")
 
+            ext = createExtent(xout)
+
             m <- leaflet::addRasterImage(map = m, x = xout[[r]], colors = p,
                                          group = grp)
             m <- mapViewLayersControl(map = m,
                                       map.types = map.types,
                                       names = grp)
 
-            if (isAvailableInLeaflet()$scl)
-              m <- leaflet::addScaleBar(map = m, position = "bottomleft")
+            m <- leaflet::addScaleBar(map = m, position = "bottomleft")
             m <- addMouseCoordinates(m)
+            m = addHomeButton(m, ext, layer.name = layer.name)
+
+            out <- methods::new('mapview', object = list(xout), map = m)
+
+            return(out)
+
+          }
+)
+
+
+#' @describeIn viewRGB \code{\link{stars}}
+setMethod("viewRGB", signature(x = "stars"),
+          function(x, r = 3, g = 2, b = 1,
+                   quantiles = c(0.02, 0.98),
+                   map = NULL,
+                   maxpixels = mapviewGetOption("mapview.maxpixels"),
+                   map.types = mapviewGetOption("basemaps"),
+                   na.color = mapviewGetOption("na.color"),
+                   layer.name = deparse(substitute(x,
+                                                   env = parent.frame())),
+                   method = c("bilinear", "ngb"),
+                   ...) {
+
+            method = match.arg(method)
+            m <- initMap(map, map.types, sf::st_crs(x)$proj4string)
+            # x <- rasterCheckSize(x, maxpixels)
+            xout <- starsCheckAdjustProjection(x, method)
+
+            mat <- cbind(as.vector(xout[[1]][, , r]),
+                         as.vector(xout[[1]][, , g]),
+                         as.vector(xout[[1]][, , b]))
+
+            if (!is.null(quantiles)) {
+
+              for(i in seq(ncol(mat))){
+                z <- mat[, i]
+                lwr <- stats::quantile(z, quantiles[1], na.rm = TRUE)
+                upr <- stats::quantile(z, quantiles[2], na.rm = TRUE)
+                z <- (z - lwr) / (upr - lwr)
+                z[z < 0] <- 0
+                z[z > 1] <- 1
+                mat[, i] <- z
+              }
+            } else {
+              # If there is no stretch we just scale the data between 0 and 1
+              mat <- apply(mat, 2, scales::rescale)
+            }
+
+            na_indx <- apply(mat, 1, anyNA)
+            cols <- mat[, 1]
+            cols[na_indx] <- na.color
+            cols[!na_indx] <- grDevices::rgb(mat[!na_indx, ], alpha = 1)
+            p <- function(x) cols
+
+            grp <- layer.name
+            lyrs <- paste(r, g, b, sep = ".")
+            grp <- paste(grp, lyrs, sep = "_")
+
+            ext = createExtent(sf::st_transform(sf::st_as_sfc(sf::st_bbox(x)), crs = 4326))
+
+            m <- addStarsImage(map = m, x = xout, band = r,
+                               colors = p, group = grp)
+            m <- mapViewLayersControl(map = m,
+                                      map.types = map.types,
+                                      names = grp)
+
+            m <- leaflet::addScaleBar(map = m, position = "bottomleft")
+            m <- addMouseCoordinates(m)
+            m = addHomeButton(m, ext, layer.name = layer.name)
 
             out <- methods::new('mapview', object = list(xout), map = m)
 
